<?php
/*
Plugin Name: Upload Plus
Plugin URI: none
Description: This is a enhanced version of the upload feature
Author: Bill Bellon
Author URI: http://www.ssec.wisc.edu/~billb/
*/ 

/*---------------------------------------------------------------------------
BUGS:
- reloading page under certain conditions gives an error message even though it shouldn't

TO DO LIST:
- decide whether to try to move file before or after database update and put in logic
  to recover from a failure - do an if statement that tests SQL statement AND 
  tests to see if file can be moved - if both are true, then update database and move
  file
- make sure to use is_null() instead of isset() when appropriate/needed and vice versa
- make sure I'm using compound boolean statements correctly
- change up_userfile to up_filename here and elsewhere so that everything is consistent and so 
  that I can simplify code (in up_updateDatabase())
- make response to file upload error better - show upload form instead of filelist in case of an error
- CLEAN UP / ORGANIZE / SIMPLIFY CODE AND TEST
- add links back to file list on error pages
- REFER TO NOTEBOOK AND PDA FOR ADDITIONAL TO DO ITEMS
- make sure to explicitly stop uploading of critical system files - file.php, index.*,
  .ht*, ...
- make sure that id of new uploaded file is not larger than 5 digits
- refer to $HOME/notebook/notes.txt 2005-11-18 notes for tests to finish on completed 
  plugin that test error checking/catching with the upload functionality
- do not print full path info on files that are deleted in confirmation screen
- save database trace / debug info into server log in a format that isn't multiple lines and
  that doesn't screw up the log file

SAVE FOR FUTURE VERSIONS:
- check to see if file already exists (in database) when uploading and act appropriately 
  (do this after integrating into uploadplus) - backup file to something like
   fileid-filename.ext.version2
   and rotate versions so that there are no more than X number of versions
   THEN, allow new uploaded file to overwrite existing one
- change file.php so that it automatically downloads file to computer with "fileNameInDB" filename
- add server response header so that when you click on view, you don't get cached file

DONE:
- USE $_FILES[$up_filevar]['filename'] IN PLACE OF $_POST['up_filename'] WHEN UPDATING
  DATABASE - AND REMOVE THE up_filename FIELD 
- for a file that is checked out allow only person who has checked out a file be able to check it in 
  AND to edit it's properties - for files that are checked in, allow anyone to edit properties
- add checkin capability
- see if I can make a popup window/page that has none of the admin-header and footer
  code BUT has access to the variables/functions used to determine if a user is logged
  in or not ==> ask on forum; otherwise, continue to integrate upload form into this page.
  CHANGED MY MIND - do the uploading in same window
- might want to always rename uploaded file "filenameB" to 
  fileid-filenameA.ext (and fileid-filenameA.ext.versionX) if file already exists
  NOT GOING TO DO THIS
- add file id to filename
- lock edit properties if file is checked out
- modify file.php and the rest of the code so that it knows to add 'file'.$id.'.'. to
  beginning of filename stored in database
- show only the file that was uploaded/checked in in up_receiveUpload() rather than the entire
  file list => NOT APPLICABLE
- add a single function that cleans ALL input
- make "checkin" in filelist only show up if you are the person who checked it out - otherwise,
  just put some text saying "checked out" in that column for that file if checked out by someone
  else - do the samething for delete
- make sure that someone else can't checkin someone else's file or edit or delete and test
- make error messages die so that HTML is completed (i.e., tags are closed)
- put in a confirmation in-between page for deleting files
- BUG: uploadplus doesn't show up unless user is level 8 or higher - fix
- set uploadplus level to something lower than 6
- use a real creationDate
- put navigation buttons/links back to file list on results pages
---------------------------------------------------------------------------*/

$up_filevar = 'up_userfile';
$up_requiredUserLevel = 4;
$uploadPlusVersion = 'BETA 0.9';


//====================================================================================
// FUNCTIONS: BEGIN
//====================================================================================
include_once('functions.inc');

function uploadplus_add_admin_menu_pages() {
  // not sure why I need to specify "__FILE__" for the file parameter, but that is what all of the examples
  // use
  add_management_page('UploadPlus', 'UploadPlus', $up_requiredUserLevel, __FILE__, 'up_manager');
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_manager() {
  global $user_level;

  if ($user_level < $up_requiredUserLevel) {
    up_authorizationFailure();
  }

  up_validateInput();

  if (isset($_REQUEST['up_action'])) {
    switch ($_REQUEST['up_action']) {
    case "cancelEditProperties":
          up_checkinoutFile($_REQUEST['up_id'], '');
          up_showFileList();
          break;
    case "upload":                           // a file was uploaded
          if ( isset($_REQUEST['up_id']) ) {
            up_receiveUpload($_REQUEST['up_id']);
          } else {
            up_receiveUpload();
          }
          break;
    case "showUploadForm":                  // show form for uploading a new file
          up_showUploadForm();
          break;
    case "delete":
          up_deleteFile($_REQUEST['up_id']);
          break;
    case "edit":
          up_editProperties($_REQUEST['up_id']);
          break;
    case "checkin":
          // make sure $id is set
          if ( !isset($_REQUEST['up_id']) || $_REQUEST['up_id'] == '') {
            echo '<div class="wrap">';
            up_showerror("ID is not set\n");
          }  
          up_showUploadForm($_REQUEST['up_id']);
          break;
    case "checkout":
          up_checkOut($_REQUEST['up_id'], $_REQUEST['up_filename']);
          break;
    default:
          echo '<div class="wrap">';
          up_showerror("Invalid action");
          break;
    }
  } else {
    up_showFileList();
  }

}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_checkinoutFile($id, $checkoutBy, $connection=NULL) {
  global $userdata;

  // make sure $id is set
  if ( !isset($id) || $id == '') {
    up_showerror("ID is not set\n");
  } 

  // open connection if necessary
  $needToCloseConnection = false;
  if (is_null($connection)) { // NO database connection inputted into function
    $connection = up_getConnection();
    $needToCloseConnection = true;
  }

  // make sure someone else doesn't check out/in a file checked out by someone else
  $sql = 'SELECT checkedout FROM wp_uploadplus WHERE id='.$id;
  if (!($result2 = @mysql_query($sql, $connection))) {
    up_showerror();
  }
  $row = mysql_fetch_array($result2, MYSQL_ASSOC);
  mysql_free_result($result2);
  if ($row['checkedout'] != '' && $row['checkedout'] != $userdata->user_login) {
    up_showerror('ERROR: you are incorrectly trying to checkin or out a file that is already checkedout by someone else or edit it or file has been deleted by someone else');
  }

  // check back in or out
  if ($checkoutBy != '') { 
    if ($checkoutBy != $userdata->user_login) {
      up_showerror('ERROR: requested checkout username does not match your username');
    }
    $sql = "UPDATE wp_uploadplus SET checkedout='".$checkoutBy."' WHERE id=".$id;
  } else {
    $sql = "UPDATE wp_uploadplus SET checkedout='' WHERE id=".$id;
  }

  if (!($result = @mysql_query($sql, $connection))) {
    up_showerror('ERROR: file not checked in');
  }

  // close connection
  if ($needToCloseConnection) {
    mysql_close($connection);
  }
}

// -----------------------------------------------------
function up_validateInput() {
  // $_FILES[] input is validated and cleaned in up_receiveUpload() separately

  if (isset($_REQUEST['up_id'])) {
    preg_match('/[0-9]{1,6}/', $_REQUEST['up_id'], $matches);
    if (count($matches) > 0) {
      $_REQUEST['up_id'] = $matches[0];
    }  else {
      $errors[] = 'invalid input parameter: "up_id"';
    }
  }

  if (isset($_REQUEST['up_action'])) {
    preg_match('/[a-zA-Z0-9_]{1,35}/', $_REQUEST['up_action'], $matches);
    if (count($matches) > 0) {
      $_REQUEST['up_action'] = $matches[0];
    }  else {
      $errors[] = 'invalid input parameter: "up_action"';
    }
  }

  if (isset($_REQUEST['up_filename'])) {
    preg_match('/[a-zA-Z\-_\.0-9]{1,70}/', $_REQUEST['up_filename'], $matches);
    if (count($matches) > 0) {
      $_REQUEST['up_filename'] = $matches[0];
    }  else {
      $errors[] = 'invalid input parameter: "up_filename"';
    }
  }

  if (isset($_REQUEST['up_oldfilename'])) {
    preg_match('/[a-zA-Z\-_\.0-9]{1,70}/', $_REQUEST['up_oldfilename'], $matches);
    if (count($matches) > 0) {
      $_REQUEST['up_oldfilename'] = $matches[0];
    }  else {
      $errors[] = 'invalid input parameter: "up_oldfilename"';
    }
  }

  if (isset($_REQUEST['up_editSubmitted'])) {
    preg_match('/true|false/', $_REQUEST['up_editSubmitted'], $matches);
    if (count($matches) > 0) {
      $_REQUEST['up_editSubmitted'] = $matches[0];
    }  else {
      $errors[] = 'invalid input parameter: "up_editSubmitted"';
    }
  }

  if (isset($_REQUEST['up_deleteConfirmed'])) {
    preg_match('/true|false/', $_REQUEST['up_deleteConfirmed'], $matches);
    if (count($matches) > 0) {
      $_REQUEST['up_deleteConfirmed'] = $matches[0];
    }  else {
      $errors[] = 'invalid input parameter: "up_deleteConfirmed"';
    }
  }

  if (isset($_REQUEST['up_title'])) {
    preg_match('/.{1,255}/', $_REQUEST['up_title'], $matches);
    if (count($matches) > 0) {
      $_REQUEST['up_title'] = htmlspecialchars($matches[0], ENT_QUOTES);
    }  else {
      $errors[] = 'invalid input parameter: "up_title"';
    }
  }

  if (isset($_REQUEST['up_description'])) {
    preg_match('/.{1,255}/', $_REQUEST['up_description'], $matches);
    if (count($matches) > 0) {
      $_REQUEST['up_description'] = htmlspecialchars($matches[0], ENT_QUOTES);
    }  else {
      $errors[] = 'invalid input parameter: "up_description"';
    }
  }

  if (count($errors) > 0) {
    $errormessage = "<ul>";
    for ($i = 0; $i < count($errors); $i++) {
      $errormessage = $errormessage . "<li><b>ERROR:</b> " . $errors[$i] . "</li>\n";
    }
    $errormessage = $errormessage . "</ul>";
    up_showerror($errormessage);
  }

}

// -----------------------------------------------------
function up_receiveUpload($id=NULL) {
  global $up_filevar;

  echo '<div class="wrap"><pre>'."\n";
  $errormessage = array(
       0=>"There is no error, the file uploaded with success",
       1=>"The uploaded file exceeds the upload_max_filesize directive in php.ini",
       2=>"The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form",
       3=>"The uploaded file was only partially uploaded",
       4=>"No file was uploaded",
       6=>"Missing a temporary folder");

  $uploaddir = '/var/apache/test/htdocs/wp-test/wp-content/files/';

  if ( isset($_FILES[$up_filevar]['error']) && $_FILES[$up_filevar]['error'] == UPLOAD_ERR_OK ) {
    // remove invalid characters from filename and clean up
    $_FILES[$up_filevar]['name'] = basename_($_FILES[$up_filevar]['name']);                  // strip off path
    $_FILES[$up_filevar]['name'] = trim($_FILES[$up_filevar]['name']);                                       // remove leading and ending white-space
    $_FILES[$up_filevar]['name'] = preg_replace('/[^a-zA-Z\-_\.0-9]/', '_', $_FILES[$up_filevar]['name']);  // replace UNIX hostile characters
    $_FILES[$up_filevar]['name'] = preg_replace('/_{1,}/', '_', $_FILES[$up_filevar]['name']);              // concatenate multiple "_" into one "_"
    $_FILES[$up_filevar]['name'] = up_fileNameInDB($_FILES[$up_filevar]['name']);  // remove file#####
  
    // check to see if file is valid
    $maxfilesize = 8*1048576; // 8 MB
    $allowedExtensions = array('jpeg', 'jpg', 'gif', 'png', 'doc', 'pdf', 'txt', 'ppt', 'xls', 'zip');
    $file_is_ok = up_checkfile($_FILES[$up_filevar]['name'], $allowedExtensions, $maxfilesize);

    if ( $file_is_ok[0] != 'yes' ) { // ERROR: there was a problem with file
      echo 'ERROR: The uploaded file was not valid (file_is_ok='.$file_is_ok[0].'); Reasons: ';
      for ($ii=1; $ii < count($file_is_ok); $ii++) {
        echo $file_is_ok[$ii] . ' : ';
      }
    } else { // file is OK
      if (is_null($id)) {
        $returnID = up_updateDatabase();
        up_checkId($returnID);      
      } else {
        $connection = up_getConnection();
        $returnID = up_updateDatabase($connection, $id);
        if ($returnID != $id) {
          up_showerror("ERROR: return ID and requested ID do not match for uploaded file");
        }
        mysql_close($connection);
      }

//      $uploadfile = $uploaddir . 'file' . sprintf("%05d", $returnID) . '.' . $_FILES[$up_filevar]['name'];  // REMOVE THIS AFTER DEBUGGING
      $uploadfile = up_realFileName($_FILES[$up_filevar]['name'], $returnID);  // ADD THIS IN AFTER DONE DEBUGGING
      if ( is_uploaded_file($_FILES[$up_filevar]['tmp_name']) && move_uploaded_file($_FILES[$up_filevar]['tmp_name'], $uploadfile) ) {
         echo "File is valid, and was successfully uploaded.\n";
      } else {
         echo "ERROR: There was an error and the file was not uploaded!\n";
      }
    }
  } else {
    echo "ERROR: file was not uploaded.  Reason: ".$errormessage[$_FILES[$up_filevar]['error']]."\n";
  }

  echo "</pre></div>\n";

  up_showFileList();
}

//----------------------------------------------------------------------------------
function basename_($filen) {
  // remove everything at the beginning of the string that isn't an alpha-numeric character
  $filen = preg_replace('/^[^a-zA-Z0-9]*/', '', $filen);
  
  if ( !(preg_match('/^[a-zA-Z0-9]/', $filen)) ) {
    up_showerror("ERROR: uploaded filename had fatal flaws\n");
  }
 
  return $filen;

}

//------------------------------------------------------------------
function up_checkfile($filename, $allowedExtensions, $maxfilesize=NULL) {
  global $up_filevar;
  $ok[0] = 'null';

  // CHECK FILE TYPE AND EXTENSION
  $array = explode(".", $filename);
  $nr    = count($array);
  $ext  = $array[$nr-1];
  if ( in_array($ext, $allowedExtensions) ) {
    $ok[0] = 'yes';
  } else {
    $ok[0] = 'no';
    $ok[] = 'uploaded file type is not allowed';
  }

  // CHECK FILE SIZE
  if ( $_FILES[$up_filevar]['size'] == 0 ) {  // make sure filesize is non-zero
      $ok[0] = 'no';
      $ok[] = 'uploaded file size is zero';
  }

  if(isset($maxfilesize)) {
    if($_FILES[$up_filevar]["size"] > $maxfilesize) {  // make sure file is not too large
      $ok[0] = 'no';
      $ok[] = 'uploaded file is too large';
    }
  }

  return $ok;
}
/* -----------------------------------------------------
------------------------------------------------------- */
function up_showUploadForm($id=NULL) {
  global $up_filevar;

  echo '<div class="wrap">'."\n";
  $title='';
  $description='';
  $label = 'Select a File to Upload';
  $checkinFileID = '';
  $oldFileName = '';

  if (isset($id) && is_numeric($id)) {
    $connection = up_getConnection();
    $username = up_getUserLogin();

    // READ DATA FROM DATABASE AND DISPLAY IN FORM
    $sql = 'SELECT title, description, filename FROM wp_uploadplus WHERE id='.$id." AND checkedout='".$username."'";
    if (!($result = @mysql_query($sql, $connection))) {
      up_showerror();
    }

    $num_rows = @mysql_num_rows($result);
    if ($num_rows == 1) {
      $row = mysql_fetch_array($result, MYSQL_ASSOC);
      $title = $row['title'];
      $description = $row['description'];
      $label = 'Select File to Checkin';
      $checkinFileID = '<input type="hidden" name="up_id" id="up_id" value="'.$id.'">'."\n";
      $oldFileName = '<input type="hidden" name="up_oldfilename" id="up_oldfilename" value="'.$row['filename'].'">'."\n";
    } else if ($num_rows > 1) {
       up_showerror('ERROR: Too many rows retrieved for requested ID');
    } else {
       up_showerror('ERROR: Requested file to checkin is already checked out by someone else or is not checked out or has been deleted by someone else');
    }
	
    mysql_free_result($result); 
    mysql_close($connection);
  }
?>
  <div class="up_nav"><a href="<?php print(UP_URL); ?>">&lt;&lt; Back to Manage Files</a></div>
  <h2>Upload Plus</h2>
  <h3 style="margin-bottom: 0px; padding-bottom: 0px;">Upload a file</h3>
  <form enctype="multipart/form-data" action="<?php echo UP_URL; ?>&amp;up_action=upload" method="POST" style="margin-top: 0px; padding-top: 0px;">
  <table class="uploadtable"><thead>
    <tr><th>Title</th><th>Description</th></tr></thead><tbody><tr>
      <td><input type="text" name="up_title" id="up_title" maxlength="255" size="25" value="<?php echo $title;?>"></td>
      <td><input type="text" name="up_description" id="up_description" maxlength="255" size="25" value="<?php echo $description;?>"></td>
    </tr></tbody></table>
    <input type="hidden" name="MAX_FILE_SIZE" value="8000000" />
    <?php echo $label;?>: <input name="<?php echo $up_filevar;?>" type="file" />
    <?php echo $checkinFileID; ?>
    <?php echo $oldFileName; ?>
    <input type="submit" value="Upload File" name="submit" id="submit" />
  </form>
  </div>
<?php
}

/* -----------------------------------------------------
Show the the list of uploaded files
------------------------------------------------------- */
function up_showFileList() {
  global $userdata, $uploadPlusVersion;

  $connection = up_getConnection();

?>
  <div class="wrap">
  <h2>Upload Plus <span style="font-size: 12px; color: #666666;">(Version: <?php echo $uploadPlusVersion; ?>)</span></h2>
  <a href="<?php print(UP_URL.'&amp;up_action=showUploadForm'); ?>"><input type="button" value="Upload a new file"></a>

  <h3 style="margin-bottom: 0px; padding: 0px;">Manage files</h3>
<?php
  // GET RID OF THIS 1st SQL CONNECTION AND MODIFY SQL CONNECTION BELOW TO INCLUDE 
  // LOGIC FOR DECIDING WHETHER OR NOT THERE ARE ANY RESULTS
  $sql = 'SELECT COUNT(id) FROM wp_uploadplus';
  if (!($result = @mysql_query($sql, $connection))) {
    up_showerror();
  }
  if (mysql_result($result, 0, 0) < 1):
    print('No files yet.');
  else:
?>
    <div style="text-align: center; margin: 0px; padding: 0px;"><a href="<?php print(UP_URL);?>"><input type="button" value="Refresh File List"></a></div>
    <table class="uploadtable">
    <thead>
      <tr><th>ID</th><th>Title</th><th>File Name</th><th>Description</th><th>Size (bytes)</th><th>Created</th><th>Modified</th>
          <th><!--View/Download--></th><th><!--Checkout and Edit / Checkin--></th><th><!--Edit Properties--></th>
          <th><!--Delete--></th></tr>
    </thead>
    <tbody>
<?php
    $sql = 'SELECT id, title, filename, description, created, modified, checkedout FROM wp_uploadplus ORDER BY id DESC';
	
    if (!($result = @mysql_query($sql, $connection))) {
      up_showerror();
    }
	
    $rowCount = 0;
    while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
      $isEven = ($rowCount % 2) == 0;

      $rowClass = $isEven ? " class=\"alternate\"" : "";

     if ($row["checkedout"] != '') { // checked out by someone
       print('<tr'.$rowClass.' style="color: red;">');
     } else {
       print('<tr'.$rowClass.'>');
     }
      print('<td>'.$row['id'].'</td>');
      print("<td>".$row["title"]."</td>");
      print("<td>".$row["filename"]."</td>");
      print("<td>".$row["description"]."</td>");
      print('<td>' . up_getFileSize($row['filename'], $row['id']) . '</td>');
      print("<td>".$row["created"]."</td>");
      print("<td>".$row["modified"]."</td>");
      print('<td><a href="/wp-test/wp-content/files/file.php?up_id='.$row['id'].'" target="_blank">View</a></td>');
      if ($row["checkedout"] == $userdata->user_login) { // checked out by current user
        print('<td><a href="'.UP_URL.'&amp;up_action=checkin&amp;up_id='.$row['id'].'" title="checked out by: '.$row['checkedout'].'">Checkin</a></td>');
        print('<td><a href="'.UP_URL.'&amp;up_action=edit&amp;up_id='.$row['id'].'">Edit Properties</a></td>');
        print('<td><a href="'.UP_URL.'&amp;up_action=delete&amp;up_id='.$row['id'].'">Delete</a></td>');
      } else if ($row["checkedout"] != '') { // checked out by someone else
        print('<td><span title="checked out by: '.$row['checkedout'].'" style="color: #bbbbbb;">Checked out</span></td>');        
        print('<td><span title="disabled - file is checked out" style="color: #bbbbbb;">Edit Properties</span></td>');
        print('<td><span title="disabled - file is checked out" style="color: #bbbbbb;">Delete</span></td>');
/*
        print('<td><a href="'.UP_URL.'&amp;up_action=checkin&amp;up_id='.$row['id'].'" title="checked out by: '.$row['checkedout'].'">Checkin (DEBUG)</a></td>');
        print('<td><a href="'.UP_URL.'&amp;up_action=edit&amp;up_id='.$row['id'].'">Edit Properties (DEBUG)</a></td>');
        print('<td><a href="'.UP_URL.'&amp;up_action=delete&amp;up_id='.$row['id'].'">Delete (DEBUG)</a></td>');
*/
      } else { // checked in
        print('<td><a href="' . UP_URL . '&amp;up_action=checkout&amp;up_id='.$row['id'].
              '&amp;up_filename='.$row['filename'].'">Checkout &amp; edit</a></td>');
        print('<td><a href="'.UP_URL.'&amp;up_action=edit&amp;up_id='.$row['id'].'">Edit Properties</a></td>');
        print('<td><a href="'.UP_URL.'&amp;up_action=delete&amp;up_id='.$row['id'].'">Delete</a></td>');
      }

      print("</tr>\n");

      $rowCount++;
    }

    mysql_free_result($result);
    mysql_close($connection);
?>
    </tbody>
    </table>
<?php
  endif;
?>
  </div>
<?php
}
/* -----------------------------------------------------
------------------------------------------------------- */
function up_deleteFile($id) {
  global $userdata;

  echo '<div class="wrap">'."\n";

  // make sure $id is set
  if ( !isset($id) || $id == '') {
    up_showerror("ID is not set\n");
  } 

  echo '<div class="up_nav"><a href="'.UP_URL.'">&lt;&lt; Back to Manage Files</a></div>'."\n";
  echo "<h2>Upload Plus: Delete File</h2>\n";

  // get info on requested file to delete and make sure it exists
  $connection = up_getConnection();
  $sql = 'SELECT filename FROM wp_uploadplus WHERE id='.$id." AND (checkedout='' OR checkedout='".$userdata->user_login."')";
  if (!($result = @mysql_query($sql, $connection))) {
    up_showerror();
  }
  if (mysql_num_rows($result) == 1) {  
    $filename = mysql_result($result, 0, 0);
    if (!up_file_exists($filename, $id)) { 
      up_showerror("requested file does not exist");
    }
  } else {  
    up_showerror("requested file to delete does not exist in database or is checked out");
  }

  // delete file or ask for confirmation
  if (!$_REQUEST['up_deleteConfirmed']) {  // delete not confirmed - ask for confirmation
    echo '<p>Are you sure you want to delete this file:<br><br>'.$filename.' ('.up_realFileName($filename, $id).' ; id='.$id.')<br><br><a href="'.UP_URL.'&amp;up_action=delete&amp;up_id='.$id.'&amp;up_deleteConfirmed=true">Yes - Delete this file</a></td>';
  } else { // delete file
    // DELETE FROM DATABASE
    $sql = 'DELETE FROM wp_uploadplus WHERE id='.$id;
    @mysql_query($sql, $connection); 
    if (@mysql_affected_rows() != 1) {
      //TO DO: rename the file back to old filename
      up_showerror("ERROR: could not delete record from database\n");
    }

    // DELETE ACTUAL FILE
    if (!up_removeFile($filename, $id)) {
      up_showerror("could not delete file");
    }

    echo '<p>The file: '.$filename.' ('.up_realFileName($filename, $id).' ; id='.$id.') has been deleted and removed from the database</p>'."\n";
  }

  mysql_free_result($result); 
  mysql_close($connection);

  echo "</div>\n";
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_editProperties($id) {
  global $userdata;

  echo '<div class="wrap">'. "\n";

  // make sure $id is set
  if ( !isset($id) || $id == '') {
    up_showerror("ID is not set\n");
  } 

  echo '<div class="up_nav"><a href="'.UP_URL.'">&lt;&lt; Back to Manage Files</a></div>'."\n";

  echo "<h2>Upload Plus: Edit Properties</h2>\n";
  $connection = up_getConnection();

  // POSSIBLY REDUNDANT CHECK
  if ($id != $_REQUEST['up_id']) {
    up_showerror("id's do not match\n");
  }

  // IF DATA HAS BEEN SUBMITTED, WRITE TO DATABASE FIRST
  if ($_REQUEST['up_editSubmitted']) {
    // check file in
    up_checkinoutFile($id, '', $connection);

    up_updateDatabase($connection, $id);
    echo "Properties for this file have been successfully updated and the file has been checked in.\n";    
  } else {
    //CHECK FILE OUT
    up_checkinoutFile($_REQUEST['up_id'], $userdata->user_login, $connection); 

    echo "<h3>File has been checked out</h3>\n";
    echo "<p>This file has been checked out and neither it or it's properties can be edited by anyone else while you have it checked out.</p>\n";

    // READ DATA FROM DATABASE AND DISPLAY IN FORM
//    $sql = 'SELECT title, filename, description, created, modified, checkedout FROM wp_uploadplus WHERE id='.$id." AND checkedout=''";
    $sql = 'SELECT title, filename, description, created, modified, checkedout FROM wp_uploadplus WHERE id='.$id;
    if (!($result = @mysql_query($sql, $connection))) {
      up_showerror();
    }

    $num_rows = @mysql_num_rows($result);
    if ($num_rows == 1) {
      $row = mysql_fetch_array($result, MYSQL_ASSOC);
      print('<form action="'.UP_URL.'&amp;up_action=edit&amp;up_id='.$id.'" method="post">'."\n");
      print ('<table><thead>
        <tr><th>ID</th><th>Title</th><th>File Name</th><th>Description</th><th>Size (bytes)</th><th>Created</th><th>Modified</th>
            <th><!--Checkout and Edit / Checkin--></th></tr></thead><tbody>');
      print("<tr>\n");
//    print('<td>'.$id.'<input type="hidden" name="up_id" id="up_id" value="'.$row['id'].'"></td>'."\n");
      print('<td>'.$id."</td>\n");
      print('<td><input type="text" name="up_title" id="up_title" value="'.$row['title'].'" maxlength="255" size="25"></td>'."\n");
      print('<td><input type="text" name="up_filename" id="up_filename" value="'.$row['filename'].'" maxlength="255" size="25"></td>'."\n");
      print('<td><input type="text" name="up_description" id="up_description" value="'.$row['description'].'" maxlength="255" size="25"></td>'."\n");
      print('<td>' . up_getFileSize($row['filename'], $row['filesize']) . '</td>'."\n");
      print("<td>".$row["created"]."</td>\n");
      print("<td>".$row["modified"]."</td>\n");
      if ($row["checkedout"] != '') { // checked out
        print('<td>Checked out by '.$row['checkedout']."</td>\n");
      } else {  // checked in
        print("<td>&nbsp;</td>\n");
      }
      print("</tr></tbody></table>\n");
      print('<div style="text-align: center; margin-top: 10px;"><input type="submit" name="up_submit_edit" value="Submit changes and check back in"> &nbsp;&nbsp; <a href="'.UP_URL.'&amp;up_action=cancelEditProperties&amp;up_id='.$id.'"><input type="button" value="Check back in without editing properties"></a></div>'."\n");
      print('<input type="hidden" name="up_editSubmitted" id="up_editSubmitted" value="true">'."\n");
      print('<input type="hidden" name="up_oldfilename" id="up_oldfilename" value="'.$row['filename'].'">'."\n");
      print("</form>\n");
    } else if ($num_rows > 1) {
       up_showerror('ERROR: Too many rows retrieved for requested ID');
    } else {
       up_showerror('ERROR: no data returned for requested ID, possibly because file is checked out by someone else, because it has been deleted by someone else or because ...');
    }
	
    mysql_free_result($result); 
  }

  mysql_close($connection);

  echo "</div>\n";

}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_updateDatabase($connection=NULL, $id=NULL) {
  global $up_filevar;
  $deletedFileName = '';

  $needToCloseConnection = false;
  if (is_null($connection)) { // NO database connection inputted into function
    $connection = up_getConnection();
    $needToCloseConnection = true;
  }

  $today = getdate();
  $creationDate=$today['year'].'-'.sprintf("%02d", $today['mon']).'-'.sprintf("%02d", $today['mday']).' '.sprintf("%02d", $today['hours']).':'.sprintf("%02d", $today['minutes']).':'.sprintf("%02d", $today['seconds']);

  if ( is_null($id) ) { // NO ID
    $sql = "INSERT INTO wp_uploadplus VALUES(NULL, '".$_REQUEST['up_title']."', '".$_FILES[$up_filevar]['name'].
           "', '".$_REQUEST['up_description']."', '".$creationDate."', NULL, '', NULL, NULL)";
  } else { // ID was supplied
    if ($_REQUEST['up_action'] == 'upload') {
      // REMINDER: change up_userfile to up_filename here and elsewhere so that everything is consistent and so 
      // that I can simplify this code
      $_FILES[$up_filevar]['name'] = up_clean($_FILES[$up_filevar]['name'], 255);
      $_FILES[$up_filevar]['name'] = addslashes($_FILES[$up_filevar]['name']);
      $_FILES[$up_filevar]['name'] = preg_replace('/^[^a-zA-Z0-9]*/', '', $_FILES[$up_filevar]['name']);
      $sql = "UPDATE wp_uploadplus SET title='".$_REQUEST['up_title']."', filename='".$_FILES[$up_filevar]['name']."', description='".
             $_REQUEST['up_description']."', modified=NULL, checkedout='' WHERE id=".$id;

      // MOVE OLD FILENAME FOR DELETION IF CHECKED IN FILENAME IS DIFFERENT
      if ($_REQUEST['up_oldfilename'] != $_FILES[$up_filevar]['name']) {
        $deletedFileName = $_REQUEST['up_oldfilename'] . '.DELETED';
        up_renameFile($_REQUEST['up_oldfilename'], $deletedFileName, $id);
      }
    } else {
      // make filename comply with filenaming conventions and escapes certain characters so that this is ready for insertion 
     // into database - last part shouldn't be necessary
      $fname = up_fileNameInDB($_REQUEST['up_filename']);
      $fname = addslashes($fname);  

/*
      $sql = "UPDATE wp_uploadplus SET title='".$_REQUEST['up_title']."', filename='".$fname."', description='".
             $_REQUEST['up_description']."' WHERE id=".$id." AND checkedout=''";*/
      $sql = "UPDATE wp_uploadplus SET title='".$_REQUEST['up_title']."', filename='".$fname."', description='".
             $_REQUEST['up_description']."' WHERE id=".$id;
      // TRY TO RENAME/MOVE FILE IF FILENAME HAS CHANGED
      if ($_REQUEST['up_oldfilename'] != $_REQUEST['up_filename']) {
        up_renameFile($_REQUEST['up_oldfilename'], $_REQUEST['up_filename'], $id);
      }
    }
  }

  @mysql_query($sql, $connection); 
  if (@mysql_affected_rows() != 1) {
    //rename the file back to old filename
    up_showerror("ERROR: properties not updated\n");
  }

  // DELETE OLD FILENAME IF CHECKED IN FILENAME IS DIFFERENT
  if ($deletedFileName != '') {
    if (up_file_exists($deletedFileName, $id) && up_removeFile($deletedFileName, $id)) { 
//    if (up_file_exists($deletedFileName, $id) && false) {   // TEST THIS AGAIN
      // come up with a better logic for this
    } else {
      up_showerror("ERROR: could not delete old version of file"); 
    } 
  }

  if ($newid = mysql_insert_id($connection)) {
    $fileid = $newid;
  } else {
    $fileid = $id;
  }

  if ($needToCloseConnection) {
    mysql_close($connection);
  }

  return $fileid;
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_checkOut($id, $filename) {
  global $userdata;
  get_currentuserinfo();

  echo '<div class="wrap">'. "\n";
  // make sure $id is set
  if ( !isset($id) || $id == '') {
    up_showerror("ID is not set\n");
  }  
  echo '<div class="up_nav"><a href="'.UP_URL.'">&lt;&lt; Back to Manage Files</a></div>'."\n";
  $connection = up_getConnection();

//   echo '<div style="font-size: 90%; margin-bottom: 10px;"><a href="'.UP_URL.'">Back to file list</a></div>'."\n";
  // TRY TO CHECK OUT FILE AND DISPLAY RESULTS
  $sql = "UPDATE wp_uploadplus SET checkedout='".$userdata->user_login."' WHERE id=" . $id . " AND checkedout=''";
  if ( @mysql_query($sql, $connection) ):
    if (@mysql_affected_rows() == 1) {
      echo "<h2>Upload Plus: File check out</h2>\n";
      echo '<p>This file has been checked out.  Download file, make your changes, then check the file back in
               by clicking on the link entitled &quot;checkin&quot; on the File List page</p>'."\n";
      echo '<p><a href="/wp-test/wp-content/files/file.php?up_id='.$id.'" target="_blank">Download file</a></p>'."\n";
    } else {
      echo '<h2>Upload Plus: File is <span style="color: red;">already checked out</span>!</h2>'."\n";
      echo "<p>This file is already checked out by someone.  Wait until this person has checked
               the file back in or contact the administrator if you encounter any problems.</p>\n";
    }
  else:
    up_showerror();
  endif;

//  mysql_free_result($result); GIVES ME AN ERROR MESSAGE ABOUT $result not being a valid resource
  mysql_close($connection);

  echo "</div>\n";
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_makefilelist($content = '') {
  $tagRegExp = '/<\!\-\-[ ]*UPLOADPLUS[ ]+ACTION="([a-zA-Z0-9 ]+)"[ ]+NUM="([a-zA-Z0-9 ]+)"[ ]*-->/i';
  if( preg_match($tagRegExp, $content, $matches) ) {
    $action=$matches[1];
    $num=$matches[2];

    if (strtolower($action) == 'filelist' && ($num >= 1 || strtolower($num) == 'all')) {
      $connection = up_getConnection();

      $sql = 'SELECT id, title, description FROM wp_uploadplus ORDER BY id DESC';

      if ($num == 'all') $num = 9999; // a really, really bad way to program, but i'm lazy
	
      if ($result = @mysql_query($sql, $connection)) {
        $rowCount = 1;
        $filelist = "<ul>\n";
        while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
          $filelist = $filelist . '<li><a href="/wp-test/wp-content/files/file.php?up_id='.$row['id'].'" title="'.$row['description'].'">'.$row['title']."</a></li>\n";
          $rowCount++;
          if ($rowCount > $num) break;
        }

        $content=preg_replace($tagRegExp, $filelist, $content);    

      }
      @mysql_free_result($result);
      @mysql_close($connection);
    }
  }

  return $content;
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_showPostForm() {
  /*
  DISABLED - NOT USED
  echo '<div class="wrap">
  <h2>UploadPlus: Upload a file</h2>
  PUT FORM FOR UPLOADING A FILE HERE AND MODIFY edit-form-advanced.php SO THAT A LINK TO
  FILE IS PUT IN POST (OR MAYBE, ADD A FIELD TO DATABASE FOR POSTS THAT LISTS ALL OF THE FILE
  ID\'S THAT ARE ATTACHED TO A POST AND PUT IN A FILTER THAT WILL GENERATE A LIST OF ATTACHED
  FILES FROM THIS FIELD.<br><br>ALSO PUT A BUTTON FOR LINKING TO A FILE THAT ALREADY IS UPLOADED
  IN THE POST.
  </div>'."\n";
  */
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_makeAttachFileMenu() {
  $connection = up_getConnection();
  $sql = 'SELECT id, filename, title, description FROM wp_uploadplus ORDER BY id DESC';

  echo '<script type="text/javascript">'."\n";
  if ($result = @mysql_query($sql, $connection)) {
    while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
      echo "edLinks[edLinks.length] = new edLink('".$row['title']."', '/wp-test/wp-content/files/file.php?up_id=".$row['id'].
            "', 0, ".$row['id'].", '".$row['filename']."');\n";
    }
  } 
  echo '</script>'."\n";
}

/* -----------------------------------------------------
------------------------------------------------------- */
function up_getUserLogin() {
  global $user_ID; 
  $user_data = get_userdata($user_ID);
  return $user_data->user_login;
}

//====================================================================================
// END OF FUNCTIONS
//====================================================================================

add_filter('the_content', 'up_makefilelist');
add_action('admin_menu', 'uploadplus_add_admin_menu_pages');

?>
